﻿---------------------
-- Himmelsscherben v0.01 --
---------------------
if HimmelsS == nil then HimmelsS = {} end
local subzone, pos


-- strings for localization
HimmelsS.moreInfo = { 
	[1] = "Stadt",
	[2] = "Solo Verlies",
	[3] = "Öffentliches Verlies",
	[4] = "Höhle", --used for Hozzin’s Folly on Bleakrock Isle
}

HimmelsS.collected = "Eingesammelt"


-- default settings for saved variables
HimmelsS.defaults = {
	showUnknown = true,
	showCollected = true,
}


-- Pins
HimmelsS.pinLayout_unknown = { 
	level = 40,
	texture = "Himmelsscherben/Icons/SkyShard-unknown.dds",
	size = 32,
}

HimmelsS.pinLayout_collected = {
	level = 40,
	texture = "Himmelsscherben/Icons/SkyShard-collected.dds",
	size = 32,
}

local function GetInfoFromTag(pin)
	local _, pinTag = pin:GetPinTypeAndTag()
	local name = GetAchievementInfo(pinTag[3]) 
	local description = "(" .. pinTag[4] ..") " .. GetAchievementCriterion(pinTag[3], pinTag[4])

	if pinTag[5] ~= nil then
		local info = "[" .. HimmelsS.moreInfo[pinTag[5]] .. "]" 
		return name, description, info
	else
		return name, description
	end
end

HimmelsS.pinTooltipCreator_unknown = {
	creator = function(pin)
		local name, description, info = GetInfoFromTag(pin)
		InformationTooltip:AddLine(name, "", ZO_HIGHLIGHT_TEXT:UnpackRGB())
		InformationTooltip:AddLine(description, "", ZO_TOOLTIP_DEFAULT_COLOR:UnpackRGB())
		if info ~= nil then
			InformationTooltip:AddLine(info, "", ZO_TOOLTIP_DEFAULT_COLOR:UnpackRGB())
		end
	end,
	tooltip = InformationTooltip,
}

HimmelsS.pinTooltipCreator_collected = {
	creator = function(pin)
		local name, description, info = GetInfoFromTag(pin)
		InformationTooltip:AddLine(name, "", ZO_HIGHLIGHT_TEXT:UnpackRGB())
		InformationTooltip:AddLine(description, "", ZO_TOOLTIP_DEFAULT_COLOR:UnpackRGB())
		if info ~= nil then
			InformationTooltip:AddLine(info .. " / [" .. HimmelsS.collected .. "]", "", ZO_TOOLTIP_DEFAULT_COLOR:UnpackRGB())
		else
			InformationTooltip:AddLine("[" .. HimmelsS.collected .. "]", "", ZO_TOOLTIP_DEFAULT_COLOR:UnpackRGB())
		end
	end,
	tooltip = InformationTooltip,
}

function HimmelsS.AddCallback_unknown(pinManager)
	if not HimmelsS.SavedVariables.showUnknown then return end

	subzone = GetMapName()
	pos = string.find (subzone, "^", 1, true)
	if pos then
		subzone = string.sub (subzone, 1, pos-1)
	end
		
	local skyshards = HimmelsS.skyshardsData[subzone]
	
	if not skyshards then return end

	for _, pinData in pairs(skyshards) do
		local _, completed = GetAchievementCriterion(pinData[3], pinData[4])
		if completed == 0 then
			pinManager:CreatePin( _G["HimmelsSMapPin_unknown"], pinData, pinData[1], pinData[2])
		end
	end
end

function HimmelsS.AddCallback_collected(pinManager)
	if not HimmelsS.SavedVariables.showCollected then return end

	subzone = GetMapName()
	pos = string.find (subzone, "^", 1, true)
	if pos then
		subzone = string.sub (subzone, 1, pos-1)
	end
	
	local skyshards = HimmelsS.skyshardsData[subzone]
	
	if not skyshards then return end

	for _, pinData in pairs(skyshards) do
		local _, completed = GetAchievementCriterion(pinData[3], pinData[4])
		if completed == 1 then
			pinManager:CreatePin( _G["HimmelsSMapPin_collected"], pinData, pinData[1], pinData[2])
		end
	end
end


-- slash commands
function HimmelsS.ShowMyPosition()
	local setMap = SetMapToPlayerLocation()
	if setMap == 2 then
		CALLBACK_MANAGER:FireCallbacks("OnWorldMapChanged")
	end

	
	local x, y = GetMapPlayerPosition("player")

	subzone = GetMapName()
	pos = string.find (subzone, "^", 1, true)
	if pos then
		subzone = string.sub (subzone, 1, pos-1)
	end

	--d(pos)
	--d(subzone)

	--local locX = string.format("%.3f", x)
	--local locY = string.format("%.3f", y)

	local locX = string.format("%03d", zo_round(x*1000))
	local locY = string.format("%03d", zo_round(y*1000))

	d(subzone .. ": " .. locX .. "," .. locY)
end

-- event handlers
function HimmelsS.OnAchievementUpdate(eventCode, achievementId)
	local ids = HimmelsS.achievementIDs

	for i = 1, #ids do
		if (ids[i] == achievementId) then
			HimmelsS.MapPins:RefreshPins()
			COMPASS_PINS:RefreshPins()
			break
		end
	end
end


function HimmelsS.OnLoad(event, name)
	if name ~= "Himmelsscherben" then return end

	HimmelsS.SavedVariables = ZO_SavedVars:New("HimmelsS_SavedVariables", 1, nil, HimmelsS.defaults)

	HimmelsS.MapPins = CustomMapPins:New()
	HimmelsS.MapPins:CreatePinType("HimmelsSMapPin_unknown", HimmelsS.pinLayout_unknown, HimmelsS.pinTooltipCreator_unknown, HimmelsS.AddCallback_unknown)
	HimmelsS.MapPins:CreatePinType("HimmelsSMapPin_collected", HimmelsS.pinLayout_collected, HimmelsS.pinTooltipCreator_collected, HimmelsS.AddCallback_collected)
	HimmelsS.MapPins:RefreshPins()
	COMPASS_PINS:RefreshPins()

	-- addon menu
	local LAM = LibStub:GetLibrary("LibAddonMenu-1.0")
	local addonMenu = LAM:CreateControlPanel("Himmelsscherben_OptionsPanel", "Himmelsscherben")

	local function GetUnknown()
		return HimmelsS.SavedVariables.showUnknown
	end

	local function SwitchUnknown()
		HimmelsS.SavedVariables.showUnknown = not HimmelsS.SavedVariables.showUnknown
		HimmelsS.MapPins:RefreshPins()
	end

	local function GetCollected()
		return HimmelsS.SavedVariables.showCollected
	end

	local function SwitchCollected()
		HimmelsS.SavedVariables.showCollected = not HimmelsS.SavedVariables.showCollected
		HimmelsS.MapPins:RefreshPins()
	end
	
	local function GetCompass()
		return HimmelsS.SavedVariables.showCompass
	end

	local function SwitchCompass()
		HimmelsS.SavedVariables.showCompass = not HimmelsS.SavedVariables.showCompass
		COMPASS_PINS:RefreshPins()
	end
	
	LAM:AddHeader(addonMenu, "Himmelsscherben_OptionsHeader", "Einstellungen")
	LAM:AddCheckbox(addonMenu, "Himmelsscherben_showUnknow", "Zeige unbekannte Himmelsscherben", "Zeige/Verberge unbekannte Himmelsscherben.", GetUnknown, SwitchUnknown)
	LAM:AddCheckbox(addonMenu, "Himmelsscherben_showCollected", "Zeige gesammelte Himmelsscherben", "Zeige/Verberge bereits eingesammelte Himmelsscherben.", GetCollected, SwitchCollected)
	LAM:AddCheckbox(addonMenu, "Himmelsscherben_showCompass", "Zeige Scherben auf dem Kompass", "Zeige/Verberge noch nicht gesammelte Himmelsscherben auf dem Kompass.", GetCompass, SwitchCompass)

	-- slash commands
	SLASH_COMMANDS["/mypos"] = HimmelsS.ShowMyPosition
	SLASH_COMMANDS["/myposition"] = HimmelsS.ShowMyPosition
	SLASH_COMMANDS["/myloc"] = HimmelsS.ShowMyPosition
	SLASH_COMMANDS["/mylocation"] = HimmelsS.ShowMyPosition

	HSInitializeCompass()
	
	EVENT_MANAGER:UnregisterForEvent("Himmelsscherben", EVENT_ADD_ON_LOADED)
end

EVENT_MANAGER:RegisterForEvent("Himmelsscherben_OnLoad", EVENT_ADD_ON_LOADED, HimmelsS.OnLoad)
EVENT_MANAGER:RegisterForEvent("Himmelsscherben_OnAchievementUpdate",  EVENT_ACHIEVEMENT_UPDATED, HimmelsS.OnAchievementUpdate)
